package zk.support.vw;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.zkoss.bind.GlobalCommandEvent;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zk.ui.event.MouseEvent;
import org.zkoss.zk.ui.select.SelectorComposer;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zul.Button;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.DefaultTreeModel;
import org.zkoss.zul.DefaultTreeNode;
import org.zkoss.zul.Label;
import org.zkoss.zul.Tree;
import org.zkoss.zul.TreeModel;
import org.zkoss.zul.TreeNode;
import org.zkoss.zul.Treecell;
import org.zkoss.zul.Treeitem;
import org.zkoss.zul.TreeitemRenderer;
import org.zkoss.zul.Treerow;

public class TreeDemoComposer extends SelectorComposer<Component> {

	@Wire("#tree")
	private Tree tree;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		
		tree.setModel(getTreeModel());
		
		tree.setItemRenderer(getRenderer());
	}
	
	
	public TreeModel<TreeNode<Payload>> getTreeModel() {
		final List<DefaultTreeNode<Payload>> children = new ArrayList<DefaultTreeNode<Payload>>();
		final DefaultTreeNode<Payload> root = new DefaultTreeNode<Payload>(
				null, children);

		System.out.println(System.currentTimeMillis());

		for (int i = 0; i < sizes.length; i++) {
			Payload p = new Payload(i + "");
			DefaultTreeNode<Payload> childNode = new DefaultTreeNode<Payload>(
					p, new ArrayList<DefaultTreeNode<Payload>>());

			for (int j = 0; j < sizes[i]; j++) {
				p = new Payload(i + "." + j);
				DefaultTreeNode<Payload> childChildNode = new DefaultTreeNode<Payload>(
						p);
				childNode.add(childChildNode);
			}

			root.add(childNode);
		}

		System.out.println(System.currentTimeMillis());

		return new DefaultTreeModel<Payload>(root) {
			int i, j, k, l; 
			
			@Override
			public boolean isLeaf(TreeNode<Payload> node) {
				i++;
				System.out.println(i + ", " + j + ", " + k + ", " + l);
				return super.isLeaf(node);
			}
			
			@Override
			public TreeNode<Payload> getChild(TreeNode<Payload> parent,
					int index) {
				j++;
				System.out.println(i + ", " + j + ", " + k + ", " + l + " parent:" + parent + index);
				return super.getChild(parent, index);
			}
			
			@Override
			public TreeNode<Payload> getChild(int[] path) {
				k++;
				System.out.println(i + ", " + j + ", " + k + ", " + l);
				return super.getChild(path);
			}
			
			@Override
			public int getChildCount(TreeNode<Payload> parent) {
				l++;
				System.out.println(i + ", " + j + ", " + k + ", " + l);
				return super.getChildCount(parent);
			}
		};
	}

	public TreeitemRenderer getRenderer() {
		
		return new TreeitemRenderer<TreeNode<Payload>>() {
			private DateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

			@Override
			public void render(Treeitem item, TreeNode<Payload> node, int index)
					throws Exception {
				
				System.out.println("render node: " + node + " " + index);
				
				item.setValue(node);
				ExtendedTreerow treerow = (ExtendedTreerow)item.getTreerow();
				if (treerow == null) {
					treerow = new ExtendedTreerow();
					treerow.setTarget(node.getData());
					item.appendChild(treerow);
				} else {
					treerow.getChildren().clear();
				}
				treerow.appendChild(cell(new Label(node.getData().getCol1())));
				treerow.appendChild(cell(dateLabel(node.getData())));
				treerow.appendChild(cell(checkbox(node.getData())));
				treerow.appendChild(cell(new Label(node.getData().getCol4())));
				treerow.appendChild(cell(new Label(node.getData().getCol5())));
				treerow.appendChild(cell(new Label(node.getData().getCol6())));
				treerow.appendChild(cell(new Label(node.getData().getCol7())));
				treerow.appendChild(cell(new Label(node.getData().getCol8())));
				treerow.appendChild(cell(editButton(node.getData())));
			}

			private Label dateLabel(Payload data) {
				Label dateLabel = new Label(format.format(data.getCol2()));
				return dateLabel;
			}

			private Component editButton(Payload data) {
				Button button = new Button("edit");
				button.setVisible(false);
				button.addEventListener(Events.ON_CLICK, new EventListener<MouseEvent>() {

					@Override
					public void onEvent(MouseEvent event) throws Exception {
						System.out.println("do some click");
					}
				});
				return button;
			}

			private Component checkbox(Payload data) {
				Checkbox checkbox = new Checkbox();
				checkbox.setChecked(data.isCol3());
				checkbox.setDisabled(true);
				return checkbox;
			}

			private Treecell cell(Component comp) {
				Treecell cell = new Treecell();
				cell.appendChild(comp);
				return cell;
			}
		};
	}

	private static int sizes[] = { 1234, 1188, 1016, 897, 602, 547, 518, 253,
			251, 236, 230, 220, 175, 167, 146, 121, 114, 108, 106, 98, 98, 97,
			93, 91, 91, 86, 84, 74, 72, 68, 68, 65, 62, 62, 62, 61, 61, 59, 57,
			57, 55, 52, 52, 51, 48, 47, 46, 44, 44, 43, 40, 40, 39, 39, 38, 37,
			37, 37, 37, 37, 37, 36, 36, 35, 35, 35, 34, 33, 33, 32, 31, 30, 30,
			30, 29, 29, 29, 28, 28, 28, 28, 28, 27, 27, 27, 27, 27, 26, 26, 26,
			26, 26, 26, 24, 24, 24, 24, 24, 24, 24, 23, 23, 23, 23, 23, 23, 23,
			23, 23, 23, 22, 22, 22, 22, 22, 21, 21, 21, 21, 21, 21, 21, 21, 21,
			21, 20, 20, 20, 20, 20, 20, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19,
			18, 18, 18, 18, 18, 18, 18, 17, 17, 17, 17, 16, 16, 16, 16, 16, 15,
			15, 15, 15, 14, 14, 14, 14, 14, 14, 14, 14, 14, 13, 13, 13, 13, 13,
			13, 13, 13, 13, 13, 13, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
			12, 12, 12, 12, 12, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
			11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 10, 10, 10, 10, 10, 10,
			10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
			10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 9, 9, 9, 9, 9, 9, 9, 9, 9,
			9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
			9, 9, 9, 9, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
			8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7,
			7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6, 6,
			6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
			6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, 4,
			4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
			4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 3, 3, 3, 3, 3,
			3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
			3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
			3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
			2, 1, 1, 1, 1, 1, 1 };

}
